<?php

require_once 'sforce.php';
require_once 'db.php';

header('Content-Type: text/json; charset=utf-8');

function getId($obj)
{
	if (is_array($obj))
		return $obj['id'];
	return $obj->id;
}

function getSForce()
{
	$client = new SForce();
	$client->login();
	return $client;
}

function getDB()
{
	$db = new Database();
	$db->connect();
	return $db;
}

try
{
	switch ($_GET['f'])
	{
		# for admin interface
		case 'getNewCampaigns':
			$client = getSForce();
			$db = getDB();

			$campaigns = array_udiff($client->getCampaigns(),
				$db->getCampaigns(),
				function ($a, $b)
				{
					return strcasecmp(getId($a), getId($b));
				});
			echo json_encode(array('campaigns' => $campaigns)), "\n";
			break;
		
		case 'getSavedCampaigns':
			echo json_encode(array('campaigns' => getDB()->getCampaigns())), "\n";
			break;
		
		case 'saveCampaign':
			$client = getSForce();
			$db = getDB();

			$campaign = $client->getCampaign($_GET['id']);
			$db->storeCampaign($campaign);
			break;

		case 'deleteCampaign':
			getDB()->deleteCampaign($_GET['id']);
			break;

		case 'uploadCampaignResults':
			$client = getSForce();
			$db = getDB();

			$res = $db->getCampaignResults($_GET['id']);
			foreach ($res as $row)
			{
				// Did we create this entry ourselves?
				if ($row['enabled'] && $row['generated'])
				{
					$hourId = $client->addHours($row['volunteerId'], $row['shiftId']);
					$db->setHourId($hourId, $row['volunteerId'], $row['shiftId']);
				}

				// Do we need to delete an entry?
				if (!$row['enabled'] && !$row['generated'])
				{
					$client->removeHours($row['hoursId']);
				}
				else
				{
					$client->setHoursStatus($row['volunteerId'], $row['shiftId'], $row['checked_in']);
				}
			}
			break;

		# for client interface
		case 'checkin':
			getDB()->checkin($_GET['id'], $_GET['time']);
			break;

		case 'uncheckin':
			getDB()->uncheckin($_GET['id']);
			break;

		case 'addToShift':
			getDB()->addToShift($_GET['volunteerId'], $_GET['shiftId']);
			break;

		case 'removeFromShift':
			getDB()->removeFromShift($_GET['volunteerId'], $_GET['shiftId']);
			break;

		case 'getUpdates':
			echo json_encode(getDB()->getUpdates($_GET['since'])), "\n";
			break;

		case 'events':
			getDB()->streamEvents();
			break;

		default:
			die('Unknown operation: ' . $_GET['f']);
	}
}
catch (Exception $e)
{
	die($e->getMessage());
}

?>
