var volunteers = [];
var volunteerData = {};
var filtered = [];
var popups = [];

function setup() {
	// Configure our filter text box
	$('#filter').keyup(function(e) {
		// Ignore special keys we handle separately
		if (e.keyCode == 40 || e.keyCode == 38 || e.keyCode == 13 || e.keyCode == 27)
			return true;

		filter($('#filter').val());
	});
	$('#filter').focus();

	// Initialize the database before doing anything else
	createDB(function() {
		showVolunteersTable();
		updateVolunteersTable();
		initUpdates();
	});

	bindKeys();
}

function createDB(doneCB) {
	// $.indexedDB('checkins').deleteDatabase();return;
	return $.indexedDB('checkins', {
		'schema': {
			'1': function(txn) {
				console.log("creating schema");
				txn.createObjectStore('campaigns', {keyPath: 'id'});
				txn.createObjectStore('jobs', {keyPath: 'id'});
				txn.createObjectStore('shifts', {keyPath: 'id'});
				txn.createObjectStore('hours', {keyPath: 'id'});
				txn.createObjectStore('volunteers', {keyPath: 'id'});
				txn.createObjectStore('queue', {autoIncrement: true});
			},
		}
	}).done(doneCB);
}

function openDB() {
	return $.indexedDB('checkins');
}

function bindKeys() {
	$(document).keyup(function(e) {
		switch (e.keyCode) {
			case 27: // escape
				if (popups.length != 0)
					popups[0].cancel();
				else
					clearInput();

				focusInput();
				return false;

			case 13: // enter
				if (uki('#vpopup').visible()) {
					if (uki('#checkin_button').visible())
						uki('#checkin_button').click();
				}
				else
					uki('#vlist').click();
				return false;

			default:
				return true;
		}
	});

	$(document).keydown(function(e) {
		if (popups.length > 0)
			return true;

		var list = uki('#vlist');
		var pos = list.selectedIndex();

		switch (e.keyCode) {
			case 40: // down
				var i = Math.min(pos + 1, list.data().length - 1);
				list.selectedIndex(i);
				return false;

			case 38: // up
				var i = Math.max(0, pos - 1);
				list.selectedIndex(i);
				return false;

			default:
				return true;
		}
	});
}

function clearInput() {
	$('#filter').val('');
	// trigger the filtering
	$('#filter').keyup();
}

function focusInput() {
	$('#filter').focus();
}

function initUpdates() {
	if (window.EventSource) {
		var source = new EventSource('ajax?f=events');
		source.addEventListener('update', function(e) {
			processUpdates(JSON.parse(e.data));
		});
		source.addEventListener('keepalive', function(e) {
			console.log('keepalive');
		});
	}

	getMaxLastUpdated(function(max) {
		getUpdates(max);
	});
}

function getUpdates(since) {
	$.ajax('ajax', {
		data: {f: 'getUpdates', since: since},
		success: processUpdates,
	});
}

function processUpdates(data) {
	var tables = ['campaigns', 'jobs', 'shifts', 'hours', 'volunteers'];

	// Empty transactions result in an error, so don't start one if we have
	// nothing to process.
	var abort = true;
	for (var i in tables) {
		if (data[tables[i]].length > 0)
			abort = false;
	}

	if (abort)
		return;

	openDB().transaction(tables, 'readwrite').then(function() {
		updateVolunteersTable();
	}, function(err, e) {
		alert("error! " + err);
	}, function (txn) {
		putUpdateResults(txn, 'campaigns', data.campaigns);
		putUpdateResults(txn, 'jobs', data.jobs);
		putUpdateResults(txn, 'shifts', data.shifts);
		putUpdateResults(txn, 'hours', data.hours);
		putUpdateResults(txn, 'volunteers', data.volunteers);

		removeDuplicateHours(data.hours);
	});
}

function putUpdateResults(txn, storeName, results) {
	if (!results || results.length < 1)
		return;

	var store = txn.objectStore(storeName, 'readwrite');
	$.each(results, function(i) {
		store.put(results[i]);
	});
}

function removeDuplicateHours(hours) {
	openDB().transaction('hours', 'readwrite').progress(function(txn) {
		for (var i in hours) {
			var key = hours[i].volunteerId + '-' + hours[i].shiftId;
			console.log('trying to delete ' + key);
			txn.objectStore('hours').delete(key);
		}
	});
}

function getMaxLastUpdated(cb) {
	var max = 0;
	var tables = ['campaigns', 'jobs', 'shifts', 'hours', 'volunteers'];
	openDB().transaction(tables, 'readonly').progress(function(txn) {
		for (var i in tables) {
			var table = tables[i];
			txn.objectStore(table, 'readonly').each(function(row) {
				if (row.value.last_updated > max) {
					max = row.value.last_updated;
				}
			});
		}
	}).done(function() {
		cb(max);
	});
}

function trackPopup(popup) {
	popups.splice(0, 0, popup);
}

function updateVolunteersTable() {
	var txn = openDB().transaction(['volunteers'], 'readonly');
	txn.done(function() {
		volunteers = $.map(volunteerData, function(v, i) {
			return { name: v.name, id: i, checked_in: v.checked_in };
		});

		// sort them
		volunteers.sort(function(a, b) {
			return a.name.localeCompare(b.name);
		});

		// trigger input
		$('#filter').keyup();
	});

	txn.progress(function(txn) {
		txn.objectStore('volunteers', 'readonly').each(function(i) {
			volunteerData[i.value.id] = { name: i.value.name, checked_in: i.value.checked_in };
		});
	});
}

function showVolunteersTable() {
	uki({ view: 'List', rect: '0 0 800 400', anchors: 'left top', id: 'vlist' })
		.attachTo($('#results')[0], 800, 800);
	uki('#vlist').click(function() {
		if (! uki('#vlist').selectedRow())
			return;

		var id = uki('#vlist').selectedIndex();
		trackPopup(new VolunteerPopup(filtered[id].id).show()
			.onClose(function() {
				popups.shift();
			}));
	});
}

function filter(str) {
	filtered = volunteers;
	var tokens = str.split(' ');
	for (var t in tokens) {
		filtered = $.grep(filtered, function(v, i) {
			return v.name.toLowerCase().indexOf(tokens[t].toLowerCase()) >= 0;
		});
	}

	uki('#vlist').data($.map(filtered, function(v, i) {
		return v.name;
	}));
	uki('#vlist').selectedIndex(str ? 0 : -1);
}

function getShiftsForVolunteer(id, cb) {
	var shifts = [];

	openDB().transaction(['hours', 'volunteers', 'shifts'], 'readonly').progress(function(txn) {
		txn.objectStore('hours').each(function(hour) {
			if (hour.value.enabled == 1 && hour.value.volunteerId == id) {
				txn.objectStore('shifts').get(hour.value.shiftId).done(function(shift) {
					shifts.push(shift);
				});
			}
		});
	})
	.done(function() {
		cb(shifts);
	});
}

function VolunteerPopup(id) {
	this.volunteerId = id;
	this.onCloseCB = null;
	var obj = this;

	this.show = function() {
		obj.showPopup();
		obj.refresh();
		return obj;
	}

	this.onClose = function(cb) {
		obj.onCloseCB = cb;
		return obj;
	}

	this.refresh = function() {
		uki('#name_val').html(volunteerData[id].name);

		// show/hide checkin buttons
		if (volunteerData[id].checked_in) {
			uki('#checkin_button').visible(false);
			uki('#uncheckin_button').visible(true);
		} else {
			uki('#checkin_button').visible(true);
			uki('#uncheckin_button').visible(false);
		}

		getShiftsForVolunteer(id, function(shifts) {
			shifts.sort(function(a, b) {
				return a.start - b.start;
			});

			var table = '<table>';
			$.each(shifts, function(i, shift) {
				table += '<tr>' +
					'<td>' + timeFormat(shift.start) + '</td>' +
					'<td>' + shift.description + '</td>' +
					// '<td>' + shift.duration + '</td>' +
					'</tr>';
			});
			table += '</table>';

			uki('#shifts_val').html(table);
		});
	}

	this.showPopup = function() {
		// Don't create multiple objects; uki has no destroy!
		if (uki('#vpopup').length >= 1) {
			uki('#vpopup').show();
			$('#filter').blur();
			return;
		}

		uki({
			view: 'Popup', rect: '0 0 600 300', anchors: '', relativeTo: uki('#vlist')[0], childViews: [
				{ view: 'Label', rect: '40 20 10 10', anchors: 'top right', html: '<p class="label">Name:</p>' },
				{ view: 'Label', rect: '40 70 10 10', anchors: 'top right', html: '<p class="label">Shifts:</p>' },
				{ view: 'Label', rect: '150 10 240 50', anchors: 'top left', id: 'name_val' },
				{ view: 'Label', rect: '150 70 340 200', anchors: 'top left', id: 'shifts_val' },
				{ view: 'Image', rect: '171 240 157 40', anchors: 'top left', id: 'checkin_button', src: 'images/unchecked.png' },
				{ view: 'Image', rect: '171 240 157 40', anchors: 'top left', id: 'uncheckin_button', src: 'images/checked.png' },
				{ view: 'Button', rect: '510 270 80 20', anchors: 'top left', id: 'reassign_button', text: 'Reassign' },
			],
			id: 'vpopup',
		}).show();

		uki('#checkin_button').click(function() {
			var id = uki('#vlist').selectedIndex();
			checkin(filtered[id].id);
		});

		uki('#uncheckin_button').click(function() {
			var id = uki('#vlist').selectedIndex();
			uncheckin(filtered[id].id);
		});

		uki('#reassign_button').click(function() {
			trackPopup(new ReassignVolunteer(obj.volunteerId, obj).show()
				.onClose(function() {
					popups.shift();
				}));
		});

		$('#filter').blur();
	}

	this.cancel = function() {
		uki('#vpopup').hide();
		if (obj.onCloseCB)
			obj.onCloseCB();
	}

	return true;
}

function ReassignVolunteer(id, parentPopup) {
	this.volunteerId = id;
	this.parentPopup = parentPopup;
	this.originalShifts = [];
	this.currentShifts = [];
	this.newShifts = [];
	this.onCloseCB = null;
	var obj = this;

	this.show = function() {
		obj.showPopup();
		obj.refresh();
		return obj;
	}

	this.onClose = function(cb) {
		obj.onCloseCB = cb;
		return obj;
	}

	this.showPopup = function() {
		if (uki('#reassign').length >= 1) {
			uki('#reassign').show();
			return;
		}

		uki({
			view: 'Popup', rect: '0 0 410 400', anchors: '', relativeTo: uki('#vpopup')[0], childViews: [
				{ view: 'Button', rect: '10 20 60 20', anchors: 'top right', id: 'shift_add_button', text: 'Add' },
				{ view: 'Button', rect: '10 50 60 20', anchors: 'top right', id: 'shift_remove_button', text: 'Remove' },
				{ view: 'Label', rect: '85 10 30 15', anchors: 'top left', html: 'Current' },
				{ view: 'ScrollPane', rect: '80 30 160 345', anchors: 'top left right bottom', childViews:
					{ view: 'List', rect: '160 340', anchors: 'top left', id: 'current_shifts' },
				},
				{ view: 'Label', rect: '250 10 30 15', anchors: 'top left', html: 'New shifts' },
				{ view: 'ScrollPane', rect: '245 30 160 345', anchors: 'top left right bottom', childViews:
					{ view: 'List', rect: '160 340', anchors: 'top left', id: 'new_shifts' },
				},
				{ view: 'Button', rect: '10 375 60 20', anchors: 'top left', id: 'reassign_cancel_button', text: 'Cancel' },
				{ view: 'Button', rect: '345 375 60 20', anchors: 'top left', id: 'reassign_save_button', text: 'Save' },
			],
			id: 'reassign'
		}).show();

		uki('#shift_add_button').click(function() {
			obj.addShift(uki('#new_shifts').selectedIndex());
		});

		uki('#shift_remove_button').click(function() {
			obj.removeShift(uki('#current_shifts').selectedIndex());
		});

		uki('#reassign_cancel_button').click(function() {
			obj.cancel();
		});

		uki('#reassign_save_button').click(function() {
			obj.save();
		});
	}

	this.refresh = function() {
		getShiftsForVolunteer(obj.volunteerId, function(shifts) {
			// do deep copies to preserve
			obj.currentShifts = shifts.slice(0);
			obj.originalShifts = shifts.slice(0);

			var names = $.map(shifts, function(val, i) {
				return val.description;
			});
			uki('#current_shifts').data(names);

			obj.getNewShifts(shifts, function(shifts) {
				obj.newShifts = shifts.slice(0);
				var names = $.map(shifts, function(val, i) {
					return val.description;
				});
				uki('#new_shifts').data(names);

				console.log('current:');
				console.log(obj.currentShifts);
				console.log('original:');
				console.log(obj.originalShifts);
				console.log('new:');
				console.log(obj.newShifts);

				console.log('current names:');
				console.log(uki('#current_shifts').data());
				console.log('new names:');
				console.log(names);
			});
		});
	}

	this.save = function() {
		console.log('saving');

		var addedShifts = [];
		var removedShifts = [];

		for (var i in obj.currentShifts) {
			var shift = obj.currentShifts[i];
			if (! obj.hasShift(obj.originalShifts, shift))
				addedShifts.push(shift.id);
		}
		for (var i in obj.originalShifts) {
			var shift = obj.originalShifts[i];
			if (! obj.hasShift(obj.currentShifts, shift))
				removedShifts.push(shift.id);
		}

		addAndRemoveShifts(obj.volunteerId, addedShifts, removedShifts, function() {
			console.log('done saving');
			uki('#reassign').hide();

			if (obj.onCloseCB)
				obj.onCloseCB();
			if (obj.parentPopup)
				obj.parentPopup.refresh();
		});
	}
	
	this.cancel = function() {
		uki('#reassign').hide();
		if (obj.onCloseCB)
			obj.onCloseCB();
	}

	this.addShift = function(i) {
		if (typeof i === 'undefined' || i < 0)
			return;

		console.log('adding: ' + i);
		console.log('before:');
		console.log('new:');
		console.log(obj.newShifts);
		console.log('current:');
		console.log(obj.currentShifts);

		var shift = obj.newShifts.splice(i, 1)[0];
		obj.currentShifts.push(shift);
		
		console.log('spliced out shift:');
		console.log(shift);

		uki('#new_shifts')[0].removeRow(i);
		uki('#current_shifts')[0].addRow(uki('#current_shifts')[0].data().length, shift.description);

		console.log('after:');
		console.log('new:');
		console.log(obj.newShifts);
		console.log('current:');
		console.log(obj.currentShifts);
	}

	this.removeShift = function(i) {
		if (typeof i === 'undefined' || i < 0)
			return;

		console.log('removing: ' + i);
		console.log('before:');
		console.log('new:');
		console.log(obj.newShifts);
		console.log('current:');
		console.log(obj.currentShifts);

		var shift = obj.currentShifts.splice(i, 1)[0];
		obj.newShifts.push(shift);
		console.log('spliced out shift:');
		console.log(shift);

		uki('#current_shifts')[0].removeRow(i);
		uki('#new_shifts')[0].addRow(uki('#new_shifts')[0].data().length, shift.description);

		console.log('after:');
		console.log('new:');
		console.log(obj.newShifts);
		console.log('current:');
		console.log(obj.currentShifts);
	}

	this.getNewShifts = function(currentShifts, cb) {
		var shifts = [];
		openDB().transaction(['shifts']).progress(function(tx) {
			tx.objectStore('shifts').each(function(shift) {
				if (! obj.hasShift(currentShifts, shift.value))
					shifts.push(shift.value);
			});
		})
		.done(function() {
			cb(shifts);
		});
	}

	this.hasShift = function(shifts, shift) {
		for (var i in shifts) {
			if (shifts[i].id == shift.id)
				return true;
		}
		return false;
	}

	return true;
}

function checkin(id) {
	var d = Math.round(new Date().getTime() / 1000);
	uki('#checkin_button').visible(false);
	uki('#uncheckin_button').visible(true);

	volunteerData[id].checked_in = d;

	openDB().transaction('volunteers', 'readwrite').progress(function(txn) {
		txn.objectStore('volunteers').get(id).done(function(volunteer) {
			volunteer.checked_in = d;
			txn.objectStore('volunteers').put(volunteer);
		});
	});

	qajax('checkin', {id: id, time: d});
}

function uncheckin(id) {
	uki('#checkin_button').visible(true);
	uki('#uncheckin_button').visible(false);

	volunteerData[id].checked_in = null;

	openDB().transaction('volunteers', 'readwrite').progress(function(txn) {
		txn.objectStore('volunteers').get(id).done(function(volunteer) {
			volunteer.checked_in = null;
			txn.objectStore('volunteers').put(volunteer);
		});
	});

	qajax('uncheckin', {id: id});
}

function addToShift(txn, volunteerId, shiftId) {
	console.log('adding shift: ' + volunteerId + ', ' + shiftId);
	var hour = {
		id: volunteerId + '-' + shiftId,
		volunteerId: volunteerId,
		shiftId: shiftId,
		enabled: 1,
	};

	txn.objectStore('hours').put(hour);
	qajaxt(txn, 'addToShift', {volunteerId: volunteerId, shiftId: shiftId});
}

function removeFromShift(txn, volunteerId, shiftId) {
	console.log('removing shift: ' + volunteerId + ', ' + shiftId);
	txn.objectStore('hours').each(function(hour) {
		if (hour.value.volunteerId == volunteerId && hour.value.shiftId == shiftId)
			hour.delete();
	});
	qajaxt(txn, 'removeFromShift', {volunteerId: volunteerId, shiftId: shiftId});
}

function addAndRemoveShifts(volunteerId, addedShifts, removedShifts, cb) {
	openDB().transaction(['hours', 'queue'], 'readwrite').progress(function(txn) {
		for (var i in addedShifts) {
			addToShift(txn, volunteerId, addedShifts[i]);
		}
		for (var i in removedShifts) {
			removeFromShift(txn, volunteerId, removedShifts[i]);
		}
	}).done(function() {
		flushQueue();
		cb();
	});
}

// Queued AJAX calls
function qajax(operation, data) {
	openDB().transaction('queue', 'readwrite').progress(function(txn) {
		qajaxt(txn, operation, data);
	}).done(flushQueue);
}

function qajaxt(txn, operation, data) {
	data.f = operation;

	txn.objectStore('queue', 'readwrite').add(data);
}

function flushQueue() {
	// Only process the first in the queue, then repeat on success.
	// If we fire them all off asynchronously, they might arrive out of
	// order or have early ones fail but later ones succeed and get
	// dequeued.

	var stop = false;
	openDB().objectStore('queue', 'readonly').each(function(i) {
		if (stop)
			return;
		stop = true;

		$.ajax('ajax', {
			data: i.value,
			success: function(data, textStatus, xhr) {
				removeQueued(i.key, flushQueue);
			},
		});
	});
}

function removeQueued(id, cb) {
	openDB().transaction('queue', 'readwrite').progress(function(txn) {
		txn.objectStore('queue', 'readwrite').delete(id);
	}).done(cb);
}

function timeFormat(timestamp) {
	if (! timestamp)
		return '';
	
	var d = new Date(timestamp * 1000);
	return new Date(timestamp * 1000).format('h:MMtt');
}

setup();
