<?php

require_once 'soapclient/SforcePartnerClient.php';
require_once 'config.php';

class SForce
{
	private $client;

	function login()
	{
		global $SFORCE_WSDL, $SFORCE_USER, $SFORCE_PASS;

		$this->client = new SforcePartnerClient();
		$this->client->createConnection($SFORCE_WSDL);
		$this->client->login($SFORCE_USER, $SFORCE_PASS);
	}

	function getCampaigns()
	{
		$campaigns = array();

		$res = $this->client->query('SELECT Id, Name, Description, StartDate FROM Campaign ORDER BY StartDate DESC');
		foreach ($res->records as $rec)
		{
			$campaigns[] = new Campaign($this->client, new SObject($rec));
		}

		return $campaigns;
	}

	function getCampaign($id)
	{
		$res = $this->client->query("SELECT Id, Name, Description, StartDate FROM Campaign WHERE Id = '$id'");
		return new Campaign($this->client, new SObject($res->records[0]));
	}

	function addHours($volunteerId, $shiftId)
	{
		$res = $this->client->query("SELECT GW_Volunteers__Volunteer_Job__c, GW_Volunteers__Start_Date_Time__c FROM GW_Volunteers__Volunteer_Shift__c WHERE Id = '$shiftId'");
		$jobId = $res->records[0]->fields->GW_Volunteers__Volunteer_Job__c;
		$start = date('Y-m-d', strtotime($res->records[0]->fields->GW_Volunteers__Start_Date_Time__c));

		$hour = new stdclass();
		$hour->type = 'GW_Volunteers__Volunteer_Hours__c';
		$hour->fields = new stdclass();
		$hour->fields->GW_Volunteers__Status__c = 'Confirmed';
		$hour->fields->GW_Volunteers__Contact__c = $volunteerId;
		$hour->fields->GW_Volunteers__Volunteer_Shift__c = $shiftId;
		$hour->fields->GW_Volunteers__Volunteer_Job__c = $jobId;
		$hour->fields->GW_Volunteers__Start_Date__c = $start;

		$res = $this->client->create(array($hour), 'GW_Volunteers__Volunteer_Hours__c');
		return $res[0]->id;
	}

	function removeHours($hourId)
	{
		$this->client->delete(array($hourId));
	}

	function setHoursStatus($volunteerId, $shiftId, $checkedIn)
	{
		$res = $this->client->query("SELECT Id, GW_Volunteers__Volunteer_Shift__r.GW_Volunteers__Duration__c FROM GW_Volunteers__Volunteer_Hours__c WHERE GW_Volunteers__Contact__c = '$volunteerId' AND GW_Volunteers__Volunteer_Shift__c = '$shiftId'");

		$hours = $res->records[0];

		if ($checkedIn)
		{
			$status = 'Completed';
			$duration = $hours->fields->GW_Volunteers__Volunteer_Shift__r->fields->GW_Volunteers__Duration__c;
		}
		else
		{
			$status = 'No-Show';
			$duration = 0;
		}

		$hours->fields = array(
			'GW_Volunteers__Status__c' => $status,
			'GW_Volunteers__Hours_Worked__c' => $duration,
		);

		$this->client->update(array($hours));
	}
}

class Campaign
{
	private $client;

	public $id;
	public $name;
	public $description;
	public $start;

	function __construct($client, $obj)
	{
		$this->client = $client;

		$this->id = $obj->Id;
		$this->name = $obj->fields->Name;
		$this->description = $obj->fields->Description;
		$this->start = $obj->fields->StartDate;
	}

	function getJobs()
	{
		$jobs = array();

		$res = $this->client->query("SELECT Id, Name, GW_Volunteers__Description__c FROM GW_Volunteers__Volunteer_Job__c WHERE GW_Volunteers__Campaign__c = '$this->id'");
		foreach ($res->records as $rec)
		{
			$jobs[] = new Job($this->client, new SObject($rec));
		}

		return $jobs;
	}
}

class Job
{
	private $client;

	public $id;
	public $name;
	public $description;

	function __construct($client, $obj)
	{
		$this->client = $client;

		$this->id = $obj->Id;
		$this->name = $obj->fields->Name;
		$this->description = $obj->fields->GW_Volunteers__Description__c;
	}

	function getShifts()
	{
		$shifts = array();

		$res = $this->client->query("SELECT Id, Name, GW_Volunteers__Description__c, GW_Volunteers__Duration__c, GW_Volunteers__Start_Date_Time__c FROM GW_Volunteers__Volunteer_Shift__c WHERE GW_Volunteers__Volunteer_Job__c = '$this->id'");
		foreach ($res->records as $rec)
		{
			$shifts[] = new Shift($this->client, new SObject($rec));
		}

		return $shifts;
	}
}

class Shift
{
	private $client;

	public $id;
	public $name;
	public $description;
	public $duration;
	public $start;

	function __construct($client, $obj)
	{
		$this->client = $client;

		$this->id = $obj->Id;
		$this->name = $obj->fields->Name;
		$this->description = $obj->fields->GW_Volunteers__Description__c;
		$this->duration = $obj->fields->GW_Volunteers__Duration__c;
		$this->start = strtotime($obj->fields->GW_Volunteers__Start_Date_Time__c);
	}

	function getHours()
	{
		$hours = array();

		$res = $this->client->query("SELECT Id, GW_Volunteers__Contact__r.Id, GW_Volunteers__Volunteer_Shift__r.Id FROM GW_Volunteers__Volunteer_Hours__c WHERE GW_Volunteers__Volunteer_Shift__r.Id = '$this->id'");
		foreach ($res->records as $rec)
		{
			$hours[] = new Hour($this->client, new SObject($rec));
		}

		return $hours;
	}
}

class Hour
{
	private $client;

	public $id;
	public $shiftId;
	public $volunteerId;

	function __construct($client, $obj)
	{
		$this->client = $client;

		$this->id = $obj->Id;
		$this->shiftId = $obj->fields->GW_Volunteers__Volunteer_Shift__r->Id;
		$this->volunteerId = $obj->fields->GW_Volunteers__Contact__r->Id;
	}

	function getVolunteer()
	{
		$res = $this->client->query("SELECT Id, FirstName, LastName FROM Contact WHERE Id = '$this->volunteerId'");
		return new Volunteer($this->client, new SObject($res->records[0]));
	}
}

class Volunteer
{
	private $client;

	public $id;
	public $name;
	public $fname;
	public $lname;

	function __construct($client, $obj)
	{
		$this->client = $client;

		$this->id = $obj->Id;
		$this->fname = $obj->fields->FirstName;
		$this->lname = $obj->fields->LastName;
		$this->name = $this->fname . ' ' . $this->lname;
	}
}

?>
